class Node:
    def __init__(self, data):
        self.data = data
        self.prev = None
        self.next = None

class DoublyLinkedList:
    def __init__(self):
        self.head = None

    def isEmpty(self):
        if not self.head:
            print("List is empty")
        else:
            print("List is not empty")

    def insert_at_beginning(self, data):
        new_node = Node(data)
        if self.head is None:
            self.head = new_node
        else:
            new_node.next = self.head
            self.head.prev = new_node
            self.head = new_node
        print(f"Element {data} inserted at the beginning")

    def insert_at_end(self, data):
        new_node = Node(data)
        if self.head is None:
            self.head = new_node
        else:
            current = self.head
            while current.next:
                current = current.next
            current.next = new_node
            new_node.prev = current
        print(f"Element {data} inserted at the end")

    def insert_at_middle(self, data, position):
        if position < 0 or position > self.count():
            print("Invalid position")
            return
        elif position == 0:
            self.insert_at_beginning(data)
        else:
            new_node = Node(data)
            current = self.head
            count = 1
            while count < position  and current:
                current = current.next
                count += 1
            if current is None:
                print("Position out of range")
            else:
                new_node.next = current.next
                if current.next:
                    current.next.prev = new_node
                current.next = new_node
                new_node.prev = current
            print(f"Element {data} inserted at position {position}")

    def delete_at_beginning(self):
        if self.head is None:
            print("List is empty")
            return
        else:
            temp = self.head
            self.head = self.head.next
            if self.head:
                self.head.prev = None
            del temp
            print("Element deleted from the beginning")

    def delete_at_end(self):
        if self.head is None:
            print("List is empty")
            return
        else:
            current = self.head
            while current.next:
                current = current.next
            if current.prev:
                current.prev.next = None
            else:
                self.head = None
            del current
            print("Element deleted from the end")

    def delete_at_middle(self, position):
        if self.head is None:
            print("List is empty")
            return
        elif position <  0 or position > self.count()-1:
            print("Invalid position")
            return
        elif position == 0:
            self.delete_at_beginning()
        else:
            current = self.head
            count = 0
            while count < position and current:
                current = current.next
                count += 1
            if current is None:
                print("Position out of range")
                return
            if current.next:
                current.prev.next = current.next
                current.next.prev = current.prev
            else:
                current.prev.next = None
            del current
            print(f"Element deleted from position {position}")

    def traverse(self):
        current = self.head
        while current:
            print()
            self.display()
            print("1. Traverse Next")
            print("2. Traverse Previous")
            print("3. Exit")
            print("Current node:", current.data)
            choice = int(input("Enter your choice: "))

            if choice == 1:
                if current.next:
                    current = current.next
                else:
                    print("There is no next node")

            elif choice == 2:
                if current.prev:
                    current = current.prev
                else:
                    print("There is no previous node")

            elif choice == 3:
                break

            else:
                print("Invalid choice")

    def search(self, data):
        current = self.head
        position = 0
        while current:
            if current.data == data:
                print(f"{data} found in the list {position}")
                return True
            current = current.next
            position += 1
        print(f"{data} not found in the list")
        return False

    def display(self):
        current = self.head
        while current:
            print(f"{current.data} <-> ", end="")
            current = current.next
        print("Null")

    def count(self):
        count = 0
        current = self.head
        while current:
            count += 1
            current = current.next
        return count


print("\t Implement Doubly Linked List ADT")
dll = DoublyLinkedList()


print("\n1. Insert at beginning")
print("2. Insert at end")
print("3. Insert at position")
print("4. Delete at beginning")
print("5. Delete at end")
print("6. Delete at position")
print("7. IsEmpty")
print("8. Search")
print("9. Display")
print("10. Count")
print("11. Traverse")
print("12. Exit")

while True:
    choice = int(input("\nEnter your choice: "))
    if choice == 1:
        data = int(input("Enter data to insert at beginning: "))
        dll.insert_at_beginning(data)
        dll.display()
    elif choice == 2:
        data = int(input("Enter data to insert at end: "))
        dll.insert_at_end(data)
        dll.display()
    elif choice == 3:
        data = int(input("Enter data to insert: "))
        position = int(input("Enter position to insert at: "))
        dll.insert_at_middle(data, position)
        dll.display()
    elif choice == 4:
        dll.delete_at_beginning()
        dll.display()
    elif choice == 5:
        dll.delete_at_end()
        dll.display()
    elif choice == 6:
        position = int(input("Enter position to delete from: "))
        dll.delete_at_middle(position)
        dll.display()
    elif choice == 7:
        dll.isEmpty()
    elif choice == 8:
        data = int(input("Enter data to search: "))
        dll.search(data)
    elif choice == 9:
        dll.display()
    elif choice == 10:
        count = dll.count()
        print("Number of nodes in the list:", count)
    elif choice == 11:
        dll.traverse()
    elif choice == 12:
            break
    else:
        print("Invalid choice. Please try again.")



from binarytree import build, Node

def insert(root,key):
    if root is None:
        return Node(key)
    if key is not None:
        if key < root.value:
            root.left = insert(root.left,key)
        else:
            root.right = insert(root.right,key)
    return root
def create():
    root = None
    values = input("Enter the value for bst (use none for empty node):").split()
    for value in values:
        try:
            if value.lower() == 'none':
                key = None
            else:
                key = int(value)
            root = insert(root,key)
        except ValueError:
            print("Invalid value Enter integer value")
    return root
def size(node):
    if node is  None:
        return 0
    return 1 + size(node.left) + size(node.right)
def height(node):
    if node is None:
        return 0
    left = height(node.left)
    right = height(node.right)
    return 1 + max(left,right)
def inorder(node):
    if node is not None:
        inorder(node.left)
        print(node.value,end=" ")
        inorder(node.right)
def preorder(node):
    if node is not None:
        
        print(node.value,end=" ")
        preorder(node.left)
        preorder(node.right)
def postorder(node):
    if node is not None:
        postorder(node.left)
        postorder(node.right)
        print(node.value,end=" ")

print("Binary Tree Creation")
bst = create()
print(bst)
size = size(bst)
print("Size of the Tree:",size)
height = height(bst) - 1
print("Height of the Tree:",height)
print("\nInorder Traversal:")
inorder(bst)
print("\nPreorder Traversal:")
preorder(bst)
print("\nPostorder Traversal:")
postorder(bst)
        




class Queue:
    def __init__(self):
        self.items =[]
    def is_empty(self):
        return len(self.items) == 0
    def enqueue(self,item):
        if len(self.items) == size:
            print("Queue is Full")
        else:
            self.items.append(item)
            print("Item",item,"enqueued to Queue")
    def is_full(self):
        if len(self.items) == size:
            print("Queue is full")
        else:
            print("Queue is not full")
    def dequeue(self):
        if not self.is_empty():
            return self.items.pop(0)
        else:
            print("Queue is empty")
            return None
    def front(self):
        if not self.is_empty():
            return self.items[0]
        else:
            print("Queue is empty")
            return None
    def rear(self):
        if not self.is_empty():
            return self.items[-1]
        else:
            print("Queue is empty")
            return None
    def size(self):
        return len(self.items)
    def display(self):
        if not self.is_empty():
            print("Queue :",self.items)
        else:
            print("Queue is empty")

print("\n\t Basic operations of Queue")
print("\t********************************\n")
size = int(input("Enter the size of Queue:"))
def main():
    q = Queue()
    while True:
        print("\n1. Enqueue")
        print("2. Dequeue")
        print("3. Front")
        print("4. Rear")
        print("5. IsEmpty")
        print("6. IsNull")
        print("7. Size")
        print("8. Display")
        print("9. Exit\n")
        choice = int(input("Enter your choice:"))
        if choice == 1:
            item = input("Enter the item to Enqueue:")
            q.enqueue(item)
        elif choice == 2:
            item = q.dequeue()
            if item is not None:
                print("Dequeue Item:",item)
        elif choice == 3:
            item = q.front()
            if item is not None:
                print("Front element of Queue:",item)
        elif choice == 4:
            item = q.rear()
            if item is not None:
                print("Last Element of Queue:",item)
        elif choice == 5:
            if q.is_empty():
                print("Queue is Empty")
            else:
                print("Queue is Not Empty")
        elif choice == 6:
            if q.is_full():
                print("Queue is Full")
        elif choice == 7:
            print("Size of Queue:",q.size())
        elif choice == 8:
            q.display()
        elif choice == 9:
            print("Exiting the program")
            break
        else:
            print("Invalid Choice")

if __name__ == "__main__":
    main()
            
        



class Stack:
    def __init__(self):
        self.items = []

    def isEmpty(self):
        return self.items == []

    def push(self, item):
        self.items.append(item)
        print(f"Pushed element: {item}, Stack: {self.items}")

    def pop(self):
        if self.isEmpty():
            raise Exception("Stack is empty")
        item = self.items.pop()
        print(f"Popped element: {item}, Stack: {self.items}")
        return item

    def peek(self):
        if self.isEmpty():
            raise Exception("Stack is empty")
        return self.items[-1]


def precedence(op):
    if op == "+" or op == "-":
        return 1
    elif op == "*" or op == "/":
        return 2
    else:
        return 0


def infix_to_postfix(expression):
    stack = Stack()
    postfix = ""
    i = 0
    while i < len(expression):
        if expression[i].isdigit():
            num = ""
            while i < len(expression) and expression[i].isdigit():
                num += expression[i]
                i += 1
            postfix += num
            postfix += " "
            continue
        elif expression[i] == '(':
            stack.push(expression[i])
        elif expression[i] == ')':
            while not stack.isEmpty() and stack.peek() != '(':
                postfix += stack.pop()
                postfix += " "
            stack.pop()
        else:
            while not stack.isEmpty() and precedence(stack.peek()) >= precedence(expression[i]):
                postfix += stack.pop()
                postfix += " "
            stack.push(expression[i])
        i += 1

    while not stack.isEmpty():
        postfix += stack.pop()
        postfix += " "

    return postfix


def evaluate_postfix(expression):
    stack = Stack()
    tokens = expression.split()
    for token in tokens:
        if token.isdigit():
            stack.push(int(token))
        else:
            val2 = stack.pop()
            val1 = stack.pop()
            result = eval(f"{val1}{token}{val2}")
            stack.push(result)

    return stack.pop()


expression = input("Enter an arithmetic expression: ")
postfix = infix_to_postfix(expression)
result = evaluate_postfix(postfix)
print(f"The result of the expression is: {result}")



def power(b,e):
    if e == 0:
        return 1
    else:
        return b*pow(b,e-1)
b = int(input("Enter the base value:"))
e = int(input("Enter the exponent value:"))
res = pow(b,e)
print(f"The power of {b} ^ {e} is :",res)



class Node:
    def __init__(self, coef, exp):
        self.coefficient = coef
        self.exponent = exp
        self.next = None

class Polynomial:
    def __init__(self):
        self.head = None

    def insert_term(self, coef, exp):
        new_term = Node(coef, exp)
        if not self.head or exp > self.head.exponent:
            new_term.next = self.head
            self.head = new_term
        else:
            current = self.head
            while current.next and exp < current.next.exponent:
                current = current.next
            new_term.next = current.next
            current.next = new_term

    def display(self):
        current = self.head
        while current:
            print(f"{current.coefficient}x^{current.exponent}", end=" ")
            if current.next:
                print("+", end=" ")
            current = current.next
        print()

    def combine_like_terms(self):
        current = self.head
        while current and current.next:
            if current.exponent == current.next.exponent:
                current.coefficient += current.next.coefficient
                current.next = current.next.next
            else:
                current = current.next

def add_polynomials(poly1, poly2):
    result = Polynomial()
    current1, current2 = poly1.head, poly2.head
    while current1 or current2:
        coef1 = current1.coefficient if current1 else 0
        exp1 = current1.exponent if current1 else 0
        coef2 = current2.coefficient if current2 else 0
        exp2 = current2.exponent if current2 else 0
        if exp1 == exp2:
            result.insert_term(coef1 + coef2, exp1)
            current1 = current1.next if current1 else None
            current2 = current2.next if current2 else None
        elif exp1 > exp2:
            result.insert_term(coef1, exp1)
            current1 = current1.next if current1 else None
        else:
            result.insert_term(coef2, exp2)
            current2 = current2.next if current2 else None
    result.combine_like_terms()
    return result

def subtract_polynomials(poly1, poly2):
    result = Polynomial()
    current1, current2 = poly1.head, poly2.head
    while current1 or current2:
        coef1 = current1.coefficient if current1 else 0
        exp1 = current1.exponent if current1 else 0
        coef2 = current2.coefficient if current2 else 0
        exp2 = current2.exponent if current2 else 0
        if exp1 == exp2:
            result.insert_term(coef1 - coef2, exp1)
            current1 = current1.next if current1 else None
            current2 = current2.next if current2 else None
        elif exp1 > exp2:
            result.insert_term(coef1, exp1)
            current1 = current1.next if current1 else None
        else:
            result.insert_term(-coef2, exp2)
            current2 = current2.next if current2 else None
    result.combine_like_terms()
    return result

def multiply_polynomials(poly1, poly2):
    result = Polynomial()
    current1 = poly1.head
    while current1:
        current2 = poly2.head
        while current2:
            result.insert_term(current1.coefficient * current2.coefficient,
                               current1.exponent + current2.exponent)
            current2 = current2.next
        current1 = current1.next
    result.combine_like_terms()
    return result

def divide_polynomials(poly1, poly2):
    result = Polynomial()
    current1 = poly1.head
    while current1:
        current2 = poly2.head
        while current2:
            if current2.coefficient != 0:
                result.insert_term(current1.coefficient / current2.coefficient,
                                   current1.exponent - current2.exponent)
            current2 = current2.next
        current1 = current1.next
    result.combine_like_terms()
    return result

def display_menu():
    print("\nMenu:")
    print("1. Add Polynomials")
    print("2. Subtract Polynomials")
    print("3. Multiply Polynomials")
    print("4. Divide Polynomials")
    print("5. Exit")

def operate_on_polynomials(poly1, poly2, operation):
    print("\nFirst Polynomial:")
    poly1.display()
    print("\nSecond Polynomial:")
    poly2.display()
    if operation == 1:
        result = add_polynomials(poly1, poly2)
        print("\nResult of Addition:")
        result.display()
    elif operation == 2:
        result = subtract_polynomials(poly1, poly2)
        print("\nResult of Subtraction:")
        result.display()
    elif operation == 3:
        result = multiply_polynomials(poly1, poly2)
        print("\nResult of Multiplication:")
        result.display()
    elif operation == 4:
        result = divide_polynomials(poly1, poly2)
        print("\nResult of Division:")
        result.display()
    elif operation == 5:
        print("Exiting the program.")
        exit()
    else:
        print("Invalid operation. Please enter a valid option (1-5).")

def input_polynomial():
    poly = Polynomial()
    n = int(input("Enter the number of terms in the polynomial: "))
    for _ in range(n):
        coef = float(input("Enter the coefficient: "))
        exp = int(input("Enter the exponent: "))
        poly.insert_term(coef, exp)
    return poly

if __name__ == "__main__":
    print("\nEnter the first polynomial:")
    poly1 = input_polynomial()
    print("\nEnter the second polynomial:")
    poly2 = input_polynomial()
    while True:
        display_menu()
        choice = int(input("Enter your choice (1-5): "))
        operate_on_polynomials(poly1, poly2, choice)



def tow(n,s,a,t):
    if n == 1:
        print(f"Move disk 1 from {s} to {t}")
        return
    tow(n-1,s,a,t)
    print(f"Move disk {n} from {s} to {t}")
    tow(n-1,a,s,t)
n = int(input("Enter the number of Disk:"))
tow(n,'A','B','C')



import networkx as nx
import matplotlib.pyplot as plt
from collections import defaultdict

class Graph():
    def __init__(self):
        self.graph = defaultdict(list)
    def add_edge(self,u,v,w):
        self.graph[u].append((v,w))
        self.graph[v].append((u,w))
    def prims(self):
        MST = []
        visited = set()
        start_vertex = next(iter(self.graph))
        visited.add(start_vertex)
        min_weight = 0

        while len(visited) < len(self.graph):
            min_edge = None
            min_edge_weight = float('inf')
            for u in visited:
                for v,weight in self.graph[u]:
                    if v not in visited and weight < min_edge_weight:
                        min_edge_weight = weight
                        min_edge = (u,v,weight)
            if min_edge:
                MST.append(min_edge)
                visited.add(min_edge[1])
                min_weight += min_edge[2]
        return MST,min_weight
    def draw_graph(self,MST):
        G = nx.Graph()
        for u,v,weight in MST:
            G.add_edge(u,v,weight=weight)
        pos = nx.spring_layout(G)
        nx.draw(G,pos,with_labels = True,node_size = 2000,node_color = 'lightblue')
        labels = nx.get_edge_attributes(G,'weight')
        nx.draw_networkx_edge_labels(G,pos,edge_labels = labels)
        plt.title('Minimum Spanning Tree Using Prims Algorithm')
        plt.show()
g = Graph()
V = int(input("Enter the Number of Vertices:"))
E = int(input("Enter the Number of Edges:"))
for _ in range(E):
    edge = input().split()
    u,v,w = edge[0],edge[1],int(edge[2])
    g.add_edge(u,v,w)
MST,min_weight = g.prims()
print("\n Selected Edges")
for edge in MST:
    print(f"{edge[0]}--{edge[1]}=={edge[2]}")
rejected_edges = set()
for u in g.graph:
    for v,weight in g.graph[u]:
        if v > u and (u,v,weight) not in [e for e in MST]:
            if (u,v,weight) not in rejected_edges:
                rejected_edges.add((u,v,weight))
                print(f"Rejected Edges : {u}--{v} == {weight} (forms cycle)")
print(f"Minimum cost:{min_weight}")
g.draw_graph(MST)
            
                



def find(total,vip,un):
    eli = []
    start = vip+1
    for seat in range(start,total-un + 1):
        if (seat - start+1)%7 == 0:
            eli.append(seat)
    return eli

total = int(input("Enter the total number of seats available in stadium:"))
vip = 200
un = 400

res = find(total,vip,un)
print("The eligible seats are :")
print(res)
print("The total number of eligible seat are:",len(res))



def input_array(length):
    arr = []
    for i in range(length):
        element = int(input(f"Enter the element {i+1}:"))
        arr.append(element)
    return arr

def insert_at(arr,element,index):
    if index < 0 or index > len(arr):
        print("Invalid Index")
        return arr
    else:
        new_arr = []
        for i in range(len(arr)+1):
            if i == index:
                new_arr.append(element)
            if i < len(arr):
                new_arr.append(arr[i])
        return new_arr
def delete_at(arr,index):
    if index < 0 or index >= len(arr):
        print("Invalid Index")
        return arr
    else:
        new_arr = []
        for i in range(len(arr)):
            if i != index:
                new_arr.append(arr[i])
        return new_arr
def search(arr,ele):
    for i in range(len(arr)):
        if arr[i] == ele:
            return i
    return -1
def update_at(arr,element,index):
    if index < 0 or index > len(arr):
        print("Invalid Index")
        return arr
    else:
        new_arr = []
        for i in range(len(arr)):
            if i != index:
                new_arr.append(arr[i])
            else:
                new_arr.append(element)
        return new_arr
def traverse(arr):
    for i in range(len(arr)):
        print(arr[i],end= "")


def main():
    n = int(input("Enter the Initial Array Length:"))
    array = input_array(n)
    print(array)
    while True:
        print("\n1. Insert an element")
        print("2. Delete an element")
        print("3. Search an element")
        print("4. Update an element")
        print("5. Traverse an element")
        print("6. Exit")
        choice = int(input("Enter your choice:"))
        if choice == 1:
            element = int(input("Enter the element to insert:"))
            index = int(input("Enter the index to insert the element:"))
            array = insert_at(array,element,index)
            print(array)
        elif choice == 2:
            index = int(input("Enter the index to delete an element:"))
            array = delete_at(array,index)
            print(array)
        elif choice == 3:
            element = int(input("Enter the element ot search:"))
            index = search(array,element)
            if index != -1:
                print(f"Element found at location {index}")
            else:
                print("Element not found")
        elif choice == 4:
            element = int(input("Enter the new element:"))
            index = int(input("Enter the index to insert the element:"))
            array = update_at(array,element,index)
            print(array)
        elif choice == 5:
            traverse(array)
        elif choice == 6:
            break
        else:
            print("Invalid Choice")
if __name__ == "__main__":
    main()
    
    
    
    



class Node:
    def __init__(self, data):
        self.data = data
        self.next = None

class LinkedList:
    def __init__(self):
        self.head = None

    def display(self):
        print("\nLinked List:")
        current = self.head
        while current:
            print(current.data, end="->")
            current = current.next
        print("Null")

    def insert_at_begin(self, data):
        new_node = Node(data)
        new_node.next = self.head
        self.head = new_node

    def insert_at_end(self, data):
        new_node = Node(data)
        if self.head is None:
            self.head = new_node
            return
        last = self.head
        while last.next:
            last = last.next
        last.next = new_node

    def insert_at_position(self, data, position):
        if position == 0:
            self.insert_at_begin(data)
            return
        current = self.head
        for _ in range(position - 1):
            if current is None:
                print("Invalid Position")
                return
            current = current.next
        if current is None:
            print("Invalid Position")
            return
        new_node = Node(data)
        new_node.next = current.next
        current.next = new_node

    def delete_at_begin(self):
        if self.head is None:
            print("List is empty")
            return
        self.head = self.head.next

    def delete_at_end(self):
        if self.head is None:
            print("List is empty")
            return
        if self.head.next is None:
            self.head = None
            return
        second_last = self.head
        while second_last.next.next:
            second_last = second_last.next
        second_last.next = None

    def delete_at_position(self, position):
        if position == 0:
            if self.head is None:
                print("List is empty")
                return
            self.head = self.head.next
            return
        current = self.head
        for _ in range(position - 1):
            if current is None:
                print("Invalid Position")
                return
            current = current.next
        if current is None or current.next is None:
            print("Invalid Position")
            return
        current.next = current.next.next

    def search(self, data):
        current = self.head
        position = 0
        while current:
            if current.data == data:
                print(f"Element {data} found at position {position}")
                return
            current = current.next
            position += 1
        print(f"Element {data} not found in list")

    def countof(self):
        current = self.head
        c = 0
        while current:
            c += 1
            current = current.next
        print("No of Nodes", c)

linked_list = LinkedList()
print("\n\t Linked List Basic Operations")
print("\t ")
while True:
    print("\n1. Insert at Beginning")
    print("2. Insert at End")
    print("3. Insert at position")
    print("4. Delete at Beginning")
    print("5. Delete at End")
    print("6. Delete at position")
    print("7. Search")
    print("8. Display")
    print("9. Count of Nodes")
    print("10. Exit\n")
    choice = int(input("Enter your choice: "))
    if choice == 1:
        data = int(input("Enter data to insert at the Beginning: "))
        linked_list.insert_at_begin(data)
        linked_list.display()
    elif choice == 2:
        data = int(input("Enter data to insert at the End: "))
        linked_list.insert_at_end(data)
        linked_list.display()
    elif choice == 3:
        data = int(input("Enter data to insert: "))
        position = int(input("Enter position to insert: "))
        linked_list.insert_at_position(data, position)
        linked_list.display()
    elif choice == 4:
        linked_list.delete_at_begin()
        linked_list.display()
    elif choice == 5:
        linked_list.delete_at_end()
        linked_list.display()
    elif choice == 6:
        position = int(input("Enter position to delete: "))
        linked_list.delete_at_position(position)
        linked_list.display()
    elif choice == 7:
        data = int(input("Enter data to search: "))
        linked_list.search(data)
    elif choice == 8:
        linked_list.display()
    elif choice == 9:
        linked_list.countof()
    elif choice == 10:
        break
    else:
        print("Invalid Choice")



def asc_desc(n):
    if n > 0 :
        print(n, end = " ")
        asc_desc(n-1)
        print(n,end = " ")

n = int(input("Enter the Natural Number:"))
print("The Ascending and Descending order are:")
asc_desc(n)



class Node:
    def __init__(self,value):
        self.value = value
        self.left = None
        self.right = None
class Tree:
    def __init__(self):
        self.root = None

    def insert(self,value):
        if not self.root:
            self.root = Node(value)
            print(f"\nRoot node inserted {value}")
            return
        queue = [self.root]
        while queue:
            current  = queue.pop(0)
            if not current.left:
                current.left = Node(value)
                print(f"\nNode {value} inserted as Child node of {current.value} (left)")
                break
            elif not current.right:
                current.right = Node(value)
                print(f"\nNode {value} inserted as Child node of {current.value} (right)")
                break
            else:
                queue.append(current.left)
                queue.append(current.right)
    def inorder(self,node):
        if node:
            self.inorder(node.left)
            print(node.value,end = " ")
            self.inorder(node.right)
    def preorder(self,node):
        if node:
            print(node.value,end = " ")
            self.preorder(node.left)
            self.preorder(node.right)
    def postorder(self,node):
        if node:
            self.postorder(node.left)
            self.postorder(node.right)
            print(node.value,end = " ")
def main():
    tree = Tree()
    print("DFS TRAVERSAL")
    print("\n1. Insert elements")
    print("2. Inorder Traversal")
    print("3. Preorder Traversal")
    print("4. PostOrder Traversal")
    print("5. Exit")
    while True:
        choice = int(input("\nEnter your choice:"))
        if choice == 1:
            value = input("Enter the value to insert:")
            tree.insert(value)
        elif choice == 2:
            print("\nInorder Traversal:")
            tree.inorder(tree.root)

        elif choice == 3:
            print("\nPreorder Traversal:")
            tree.preorder(tree.root)
        elif choice == 4:
            print("\nPostorder Traversal:")
            tree.postorder(tree.root)
        elif choice == 5:
            break
        else:
            print("Invalid Choice")
if __name__ == "__main__":
    main()




def fib(n):
    if n <= 1:
        return n
    else:
        return fib(n-1) + fib(n-2)

n = int(input("Enter the number to get fibonaaci:"))
print(f"The fibonacci number for the given {n} is :")
for i in range(n):
    print(fib(i), end = " ")



import heapq
from binarytree import build

class PriorityQueue:
    def __init__(self):
        self.min_heap = []
        self.max_heap = []
    def insert(self,value):
        heapq.heappush(self.min_heap,value)
        heapq.heappush(self.max_heap,-value)
    def dequeue_min(self):
        if not self.isempty():
            min_val = heapq.heappop(self.min_heap)
            self.max_heap.remove(-min_val)
            heapq.heapify(self.max_heap)
            return min_val
    def dequeue_max(self):
        if not self.isempty():
            max_val = -heapq.heappop(self.max_heap)
            self.min_heap.remove(max_val)
            heapq.heapify(self.min_heap)
            return max_val
    def peek_min(self):
        if not self.isempty():
            return self.min_heap[0]
    def peek_max(self):
        if not self.isempty():
            return -self.max_heap[0]
    def display_min(self):
        if not self.isempty():
            tree = build(self.min_heap)
            print("Min Heap Tree")
            print(tree)
        else:
            print("Min heap tree is empty")
    def display_max(self):
        if not self.isempty():
            tree = build([-val for val in self.max_heap])
            print("Max Heap Tree")
            print(tree)
        else:
            print("Max heap tree is empty")
    def isempty(self):
        return len(self.min_heap) == 0
    def size_min(self):
        return len(self.min_heap)
    def size_max(self):
        return len(self.max_heap)
def main():
    pq = PriorityQueue()
    print("Priority Queue using Heap")
    print("\n1. Enqueue the element")
    print("2. Dequeue from Min heap")
    print("3. Dequeue from Max heap")
    print("4. Peek the min heap and max heap")
    print("5. Display the Min heap")
    print("6. Display the Max heap")
    print("7. Check if empty")
    print("8. Size of Min heap")
    print("9. Size of Max heap")
    print("10. Exit")

    while True:
        choice = int(input("Enter your choice:"))
        if choice == 1:
            value = int(input("Enter the element to enqueue"))
            pq.insert(value)
            print(f"Enqueue element {value}")
        elif choice == 2:
            min_val = pq.dequeue_min()
            if min_val is not None:
                print(f"Dequeued Element:",min_val)
            else:
                print("Min heap is empty")
        elif choice == 3:
            max_val = pq.dequeue_max()
            if max_val is not None:
                print(f"Dequeued Element:",max_val)
            else:
                print("Min heap is empty")
        elif choice == 4:
            min_val = pq.peek_min()
            max_val = pq.peek_max()
            print("The Min heap  Front element:",min_val)
            print("The Max heap  Front element:",max_val)
        elif choice == 5:
            pq.display_min()
        elif choice == 6:
            pq.display_max()
        elif choice == 7:
            if pq.isempty():
                print("Queue is empty")
            else:
                print("Queue is not empty")
        elif choice == 8:
            print("Size of Min heap:",pq.size_min())
        elif choice == 9:
            print("Size of Max heap:",pq.size_max())
        elif choice == 10:
            break
if __name__ == "__main__":
    main()
            
            
        
    



class Stack():
    def __init__(self):
        self.item = []
    def empty(self):
        return len(self.item) == 0
    def push(self,item):
        self.item.append(item)
    def pop(self):
        if not self.empty():
            return self.item.pop()
        else:
            print("Cannot Pop from empty Stack")
    def peek(self):
        if not self.empty():
            return self.item[-1]
        else:
            print("Cannot Peek in empty stack")
    def display(self):
        print("Stack:",self.item)
def main():
    s = Stack()
    print("\n Menu")
    print("1. Push")
    print("2. Pop")
    print("3. Peek")
    print("4. Display")
    print("5. Exit")
    while True:
        choice = int(input("Enter the choice:"))
        if choice == 1:
            item = int(input("Enter the item to push:"))
            s.push(item)
            s.display()
        elif choice == 2:
            pop_i = s.pop()
            if pop_i is not None:
                print("Popped Item:",pop_i)
                s.display()
            else:
                print("Stack is Empty")
        elif choice == 3:
            pp = s.peek()
            if pp is not None:
                print("Peek Element:",pp)
            else:
                print("Not peek an empty stack")
        elif choice ==4:
            s.display()
        elif choice == 5:
            break
        else:
            print("Invalid choice")
if __name__== "__main__":
    main()



def merge(a,depth = 0):
    if len(a) > 1:
        mid = len(a)//2
        left = a[:mid]
        right = a[mid:]
        merge(left,depth+1)
        merge(right,depth+1)
        print(" " * depth,'Dividing',a)
        print(" " * depth,"Merging",left,"and",right)
        a.clear()
        i = j = 0
        while i < len(left) and j < len(right):
            if left[i] <= right[j]:
                a.append(left[i])
                i += 1
            else:
                a.append(right[j])
                j += 1
        while i < len(left):
                a.append(left[i])
                i += 1
        while j < len(right):
                a.append(right[j])
                j += 1
        print(" " *depth,"Result:",a)
a = []
n = int(input("Enter the Upper Limit:"))
for i in range(n):
    ele = int(input("Enter the Elements:"))
    a.append(ele)
print("Before Sorting:",a)
merge(a)
print("Sorted Array:",a)
                



import networkx as nx
import matplotlib.pyplot as plt

class Graph:
    def __init__(self):
        self.graph = []

    def add_edge(self, u, v, w):
        self.graph.append((u, v, w))

    def find(self, parent, i):
        if parent[i] == -1:
            return i
        return self.find(parent, parent[i])

    def union(self, parent, rank, x, y):
        xroot = self.find(parent, x)
        yroot = self.find(parent, y)

        if rank[xroot] < rank[yroot]:
            parent[xroot] = yroot
        elif rank[xroot] > rank[yroot]:
            parent[yroot] = xroot
        else:
            parent[yroot] = xroot
            rank[xroot] += 1

    def kruskal(self):
        result = []
        rejected_edges = []

        self.graph = sorted(self.graph, key=lambda item: item[2])
        vertex_indices = {}
        vertices = set()
        for edge in self.graph:
            u, v, _ = edge
            vertices.add(u)
            vertices.add(v)
        parent = [-1] * len(vertices)
        rank = [0] * len(vertices)

        for edge in self.graph:
            u, v, _ = edge
            if u not in vertex_indices:
                vertex_indices[u] = len(vertex_indices)
            if v not in vertex_indices:
                vertex_indices[v] = len(vertex_indices)

        print("Sorted Edges:")
        for u, v, w in self.graph:
            print(f"{u} -- {v} == {w}")

        print("\nMinimum Spanning Tree (MST):")
        for edge in self.graph:
            u, v, w = edge
            x = self.find(parent, vertex_indices[u])
            y = self.find(parent, vertex_indices[v])

            if x != y:
                result.append([u, v, w])
                self.union(parent, rank, x, y)
                print(f"Selected edge: {u} -- {v} == {w}")
                print("Current MST edges:")
                for u, v, weight in result:
                    print(f"{u} -- {v} == {weight}")
                print()
            else:
                rejected_edges.append([u, v, w])
                print(f"Rejected edge: {u} -- {v} == {w} (Forms cycle)")

        minimum_cost = sum(w for _, _, w in result)
        print(f"\nMinimum Cost Spanning Tree: {minimum_cost}")

        
        G = nx.Graph()
        for u, v, w in result:
            G.add_edge(u, v, weight=w)

        
        pos = nx.spring_layout(G)
        nx.draw(G, pos, with_labels=True, font_weight='bold', node_size=2000, node_color='lightblue')
        labels = nx.get_edge_attributes(G, 'weight')
        nx.draw_networkx_edge_labels(G, pos, edge_labels=labels)

        plt.title('Minimum Spanning Tree with Rejected Edges')
        plt.show()


g = Graph()

V = int(input("Enter the number of vertices: "))
E = int(input("Enter the number of edges: "))

print("Enter the edges as 'u v weight':")
for _ in range(E):
    edge = input().split()
    u, v, w = edge[0], edge[1], int(edge[2])
    g.add_edge(u, v, w)

g.kruskal()



def quicksort(arr,left,right):
    if left < right:
        partition_pos = partition(arr,left,right)
        print_step(arr,partition_pos,left,right)
        quicksort(arr,left,partition_pos -1)
        quicksort(arr,partition_pos + 1,right)
def partition(arr,left,right):
    i = left
    j = right - 1
    pivot = arr[right]
    while i < j:
        while i < right and arr[i] < pivot:
            i += 1
        while j > left and arr[j] >= pivot:
            j -= 1
        if i < j:
            arr[i], arr[j] = arr[j], arr[i]
    if arr[i] > pivot:
        arr[i], arr[right] = arr[right], arr[i]
    return i
def print_step(arr,pivot_index,left,right):
    pivot = arr[pivot_index]
    print(f"Pivot : {pivot}, Left: {arr[left:pivot_index]}, Right:{arr[pivot_index+1:right+1]}")

n = int(input("Enter the upper limit:"))
arr = []
for i in range(n):
    element = int(input("Enter the elements:"))
    arr.append(element)
print("Initital array:", arr)
quicksort(arr,0,len(arr)-1)
print("Sorted Array:",arr)
        
